@@ -17,13 +17,14 @@ import orjson
 import copy
 import multiprocessing as mp
 import pathlib
-from tethysts.utils import get_object_s3, url_stream_to_file, read_pkl_zstd, b2_public_key_pattern
+from tethysts.utils import read_pkl_zstd, b2_public_key_pattern
 import gzip
 import zstandard as zstd
 import subprocess
 import glob
 import rioxarray as rxr
 import concurrent.futures
+from tethys_utils import misc
 # from cdo import Cdo
 # mp.set_start_method('spawn')
 
@@ -359,9 +360,9 @@ def copy_s3_file(source_path, dest_path, bucket, s3=None, connection_config=None
         try:
             if isinstance(public_url, str):
                 url1 = b2_public_key_pattern.format(base_url=public_url.rstrip('/'), bucket=bucket, obj_key=source_path)
-                file_path3 = url_stream_to_file(url1, file_path2, compression=compression, chunk_size=chunk_size)
+                file_path3 = misc.url_stream_to_file(url1, file_path2, compression=compression, chunk_size=chunk_size)
             else:
-                url1 = get_object_s3(source_path, connection_config=connection_config, bucket=bucket, counter=5)
+                url1 = misc.get_object_s3(source_path, connection_config=connection_config, bucket=bucket, counter=5)
                 if compression == 'zstd':
                     if file_path2.endswith('.zst'):
                         file_path2 = os.path.splitext(file_path2)[0]
