@@ -10,7 +10,7 @@ import numpy as np
 import xarray as xr
 import pandas as pd
 from tethys_utils import misc, s3, processing, titan, data_io
-from tethysts.utils import s3_client, get_object_s3, read_json_zstd, read_pkl_zstd, download_results, create_public_s3_url
+from tethysts.utils import read_json_zstd, read_pkl_zstd, download_results, create_public_s3_url
 import tethys_data_models as tdm
 from typing import List, Optional, Dict, Union
 import pandas as pd
@@ -231,7 +231,7 @@ def get_obj_list(glob_path, date_format=None, freq=None, from_date=None, to_date
         glob_path3, glob_ext = os.path.split(glob_path2)
         if not glob_path3.endswith('/'):
             glob_path3 = glob_path3 + '/'
-        client = s3_client(connection_config)
+        client = misc.s3_client(connection_config)
         obj_list = s3.list_objects_s3(client, bucket, glob_path3, delimiter='/', date_format=date_format)
         obj_list1 = obj_list.rename(columns={'Key': 'path', 'Size': 'size', 'KeyDate': 'date'}).drop(['LastModified', 'ETag'], axis=1).copy()
         # obj_list1['remote_type'] = 's3'
@@ -271,7 +271,7 @@ def copy_source_objs(source_paths, dest_path, bucket=None, connection_config=Non
     objs = source_paths
 
     if isinstance(connection_config, (dict, str)) and isinstance(bucket, str):
-        client = s3_client(connection_config, threads)
+        client = misc.s3_client(connection_config, threads)
 
         if isinstance(max_workers, int):
 
@@ -312,7 +312,7 @@ def copy_interim_objs(source_paths, dest_path, bucket, connection_config=None, p
     """
 
     """
-    client = s3_client(connection_config, threads)
+    client = misc.s3_client(connection_config, threads)
 
     path1 = pathlib.Path(dest_path)
     with concurrent.futures.ThreadPoolExecutor(max_workers=threads) as executor:
@@ -712,7 +712,7 @@ class Grid(titan.Titan):
 
         """
         with concurrent.futures.ThreadPoolExecutor(max_workers=threads) as executor:
-            client = s3_client(self.connection_config, max_pool_connections=threads)
+            client = misc.s3_client(self.connection_config, max_pool_connections=threads)
             futures = []
             for path in source_paths:
                 f = executor.submit(s3.put_interim_results_s3, client, self.bucket, path, self.run_id, system_version=self.system_version)
@@ -748,7 +748,7 @@ class Grid(titan.Titan):
         key = key_pattern.format(run_id=self.run_id)
 
         ## Query objects
-        client = s3_client(self.connection_config)
+        client = misc.s3_client(self.connection_config)
         obj_list1 = s3.list_objects_s3(client, self.bucket, key)
         obj_list1 = obj_list1[['Key', 'Size']].rename(columns={'Key': 'key', 'Size': 'size'})
 
@@ -808,7 +808,7 @@ class Grid(titan.Titan):
         """
         prefix = tdm.utils.key_patterns[self.system_version]['interim_results'].split('{dataset_id}')[0].format(run_id=self.run_id)
 
-        client = s3_client(self.connection_config)
+        client = misc.s3_client(self.connection_config)
 
         obj_list = s3.list_object_versions_s3(client, self.bucket, prefix)
 
