@@ -5,6 +5,7 @@ Created on Sat May  1 13:22:21 2021
 
 @author: mike
 """
+import io
 import os
 import socket
 import numpy as np
@@ -28,17 +29,253 @@ import pathlib
 from pydantic import HttpUrl
 from typing import Optional, List, Any, Union
 import botocore
-from tethysts.utils import s3_client, get_object_s3, create_public_s3_url, url_stream_to_file
+import gzip
+import tethys_data_models as tdm
+import copy
+import boto3
+import botocore
+import shutil
+
 
 ############################################
 ### Parameters
 
 local_results_name = '{ds_id}/{stn_id}/{chunk_id}.{version_date}.{chunk_hash}.nc'
+b2_public_key_pattern = '{base_url}/{bucket}/{obj_key}'
+contabo_public_key_pattern = '{base_url}:{bucket}/{obj_key}'
+public_remote_key = 'https://b2.tethys-ts.xyz/file/tethysts/tethys/public_remotes_v4.json.zst'
 
 ############################################
 ### Functions
 
 
+def create_public_s3_url(base_url, bucket, obj_key):
+    """
+    This should be updated as more S3 providers are added!
+    """
+    if 'contabo' in base_url:
+        key = contabo_public_key_pattern.format(base_url=base_url.rstrip('/'), bucket=bucket, obj_key=obj_key)
+    else:
+        key = b2_public_key_pattern.format(base_url=base_url.rstrip('/'), bucket=bucket, obj_key=obj_key)
+
+    return key
+
+
+class ResponseStream(object):
+    """
+    In many applications, you'd like to access a requests response as a file-like object, simply having .read(), .seek(), and .tell() as normal. Especially when you only want to partially download a file, it'd be extra convenient if you could use a normal file interface for it, loading as needed.
+This is a wrapper class for doing that. Only bytes you request will be loaded - see the example in the gist itself.
+https://gist.github.com/obskyr/b9d4b4223e7eaf4eedcd9defabb34f13
+    """
+    def __init__(self, request_iterator):
+        self._bytes = io.BytesIO()
+        self._iterator = request_iterator
+
+    def _load_all(self):
+        self._bytes.seek(0, io.SEEK_END)
+        for chunk in self._iterator:
+            self._bytes.write(chunk)
+
+    def _load_until(self, goal_position):
+        current_position = self._bytes.seek(0, io.SEEK_END)
+        while current_position < goal_position:
+            try:
+                current_position += self._bytes.write(next(self._iterator))
+            except StopIteration:
+                break
+
+    def tell(self):
+        return self._bytes.tell()
+
+    def read(self, size=None):
+        left_off_at = self._bytes.tell()
+        if size is None:
+            self._load_all()
+        else:
+            goal_position = left_off_at + size
+            self._load_until(goal_position)
+
+        self._bytes.seek(left_off_at)
+        return self._bytes.read(size)
+
+    def seek(self, position, whence=io.SEEK_SET):
+        if whence == io.SEEK_END:
+            self._load_all()
+        else:
+            self._bytes.seek(position, whence)
+
+
+def s3_client(connection_config: dict, max_pool_connections: int = 30):
+    """
+    Function to establish a client connection with an S3 account. This can use the legacy connect (signature_version s3) and the curent version.
+    Parameters
+    ----------
+    connection_config : dict
+        A dictionary of the connection info necessary to establish an S3 connection. It should contain service_name, endpoint_url, aws_access_key_id, and aws_secret_access_key. connection_config can also be a URL to a public S3 bucket.
+    max_pool_connections : int
+        The number of simultaneous connections for the S3 connection.
+    Returns
+    -------
+    S3 client object
+    """
+    ## Validate config
+    _ = tdm.base.ConnectionConfig(**connection_config)
+
+    s3_config = copy.deepcopy(connection_config)
+
+    if 'config' in s3_config:
+        config0 = s3_config.pop('config')
+        config0.update({'max_pool_connections': max_pool_connections})
+        config1 = boto3.session.Config(**config0)
+
+        s3_config1 = s3_config.copy()
+        s3_config1.update({'config': config1})
+
+        s3 = boto3.client(**s3_config1)
+    else:
+        s3_config.update({'config': botocore.config.Config(max_pool_connections=max_pool_connections)})
+        s3 = boto3.client(**s3_config)
+
+    return s3
+
+
+def get_object_s3(obj_key: str, bucket: str, s3: botocore.client.BaseClient = None, connection_config: dict = None, public_url: HttpUrl=None, version_id=None, range_start: int=None, range_end: int=None, counter=5):
+    """
+    General function to get an object from an S3 bucket. One of s3, connection_config, or public_url must be used.
+    Parameters
+    ----------
+    obj_key : str
+        The object key in the S3 bucket.
+    s3 : botocore.client.BaseClient
+        An S3 object created via the s3_client function.
+    connection_config : dict
+        A dictionary of the connection info necessary to establish an S3 connection. It should contain service_name, s3, endpoint_url, aws_access_key_id, and aws_secret_access_key.
+    public_url : str
+        A URL to a public S3 bucket. This is generally only used for Backblaze object storage.
+    bucket : str
+        The bucket name.
+    counter : int
+        Number of times to retry to get the object.
+    Returns
+    -------
+    bytes
+        bytes object of the S3 object.
+    """
+    counter1 = counter
+
+    get_dict = {'Key': obj_key, 'Bucket': bucket}
+
+    if isinstance(version_id, str):
+        get_dict['VersionId'] = version_id
+
+    ## Range
+    range_dict = {}
+
+    if range_start is not None:
+        range_dict['start'] = str(range_start)
+    else:
+        range_dict['start'] = ''
+
+    if range_end is not None:
+        range_dict['end'] = str(range_end)
+    else:
+        range_dict['end'] = ''
+
+    if range_dict:
+        get_dict['Range'] = 'bytes={start}-{end}'.format(**range_dict)
+
+    ## Get the object
+    while True:
+        try:
+            if isinstance(public_url, str) and (version_id is None):
+                url = create_public_s3_url(public_url, bucket, obj_key)
+                resp = requests.get(url, timeout=300)
+                resp.raise_for_status()
+
+                ts_obj = resp.content
+
+            elif isinstance(s3, botocore.client.BaseClient):
+                ts_resp = s3.get_object(**get_dict)
+                ts_obj = ts_resp.pop('Body').read()
+
+            elif isinstance(connection_config, dict):
+                ## Validate config
+                _ = tdm.base.ConnectionConfig(**connection_config)
+
+                s3 = s3_client(connection_config)
+
+                ts_resp = s3.get_object(**get_dict)
+                ts_obj = ts_resp.pop('Body').read()
+            else:
+                raise TypeError('One of s3, connection_config, or public_url needs to be correctly defined.')
+            break
+        except:
+            # print(traceback.format_exc())
+            if counter1 == 0:
+                # raise ValueError('Could not properly download the object after several tries')
+                print('Object could not be downloaded.')
+                return None
+            else:
+                # print('Could not properly extract the object; trying again in 5 seconds')
+                counter1 = counter1 - 1
+                sleep(5)
+
+    return
+
+
+def url_stream_to_file(url, file_path, compression=None, chunk_size=524288):
+    """
+    """
+    file_path1 = pathlib.Path(file_path)
+    if file_path1.is_dir():
+        file_name = url.split('/')[-1]
+        file_path2 = str(file_path1.joinpath(file_name))
+    else:
+        file_path2 = file_path
+
+    base_path = os.path.split(file_path2)[0]
+    os.makedirs(base_path, exist_ok=True)
+
+    counter = 4
+    while True:
+        try:
+            with requests.get(url, stream=True, timeout=300) as r:
+                r.raise_for_status()
+                stream = ResponseStream(r.iter_content(chunk_size))
+
+                if compression == 'zstd':
+                    if str(file_path2).endswith('.zst'):
+                        file_path2 = os.path.splitext(file_path2)[0]
+                    dctx = zstd.ZstdDecompressor()
+
+                    with open(file_path2, 'wb') as f:
+                        dctx.copy_stream(stream, f, read_size=chunk_size, write_size=chunk_size)
+
+                elif compression == 'gzip':
+                    if str(file_path2).endswith('.gz'):
+                        file_path2 = os.path.splitext(file_path2)[0]
+
+                    with gzip.open(stream, 'rb') as s_file, open(file_path2, 'wb') as d_file:
+                        shutil.copyfileobj(s_file, d_file, chunk_size)
+                else:
+                    with open(file_path2, 'wb') as f:
+                        chunk = stream.read(chunk_size)
+                        while chunk:
+                            f.write(chunk)
+                            chunk = stream.read(chunk_size)
+
+                break
+
+        except Exception as err:
+            if counter < 1:
+                raise err
+            else:
+                counter = counter - 1
+                sleep(5)
+
+    return file_path2
+
+
 def path_date_parser(path, date_format=None):
     """
 
