@@ -10,8 +10,8 @@ import numpy as np
 import pandas as pd
 import orjson
 import tethys_data_models as tdm
-from tethysts.utils import get_object_s3, read_json_zstd, s3_client, read_pkl_zstd
-from tethys_utils.misc import make_run_date_key, path_date_parser, write_json_zstd
+from tethysts.utils import read_json_zstd, read_pkl_zstd
+from tethys_utils.misc import make_run_date_key, path_date_parser, write_json_zstd, get_object_s3, s3_client
 from tethysts import Tethys
 from botocore import exceptions as bc_exceptions
 from time import sleep
