@@ -44,7 +44,9 @@ data_path1 = '/media/sdb1/Data/ecmwf/era5-land/2m_temperature_2003-2013_reanalys
 data_path2 = '/media/sdb1/Data/ecmwf/era5-land/2m_temperature_*.nc'
 data_path4 = '/media/sdb1/Data/ecmwf/era5-land/2m_temperature_2003-2013_reanalysis-era5-land_*.nc'
 data_path3 = '/media/sdb1/Data/ecmwf/era5-land/2m_temperature-test1.nc'
-data_path = '/media/sdb1/Data/ecmwf/era5-land/'
+data_path4 = '/media/sdb1/Data/ecmwf/era5-land/test/*.nc'
+data_path5 = '/media/sdb1/Data/ecmwf/era5-land/test/total_evaporation_1981-1991_reanalysis-era5-land.nc'
+data_path = ['/media/sdb1/Data/ecmwf/era5-land/test/2m_temperature_*.nc', '/media/sdb1/Data/ecmwf/era5-land/test/total_evaporation_*.nc']
 parameter_code = 'temp_at_2'
 
 grid = data_path2
@@ -61,17 +63,17 @@ samples = 900
 ### Tests
 
 
-def preprocessor(ds):
-    """
+# def preprocessor(ds):
+#     """
 
-    """
-    ds = ds.rename({'longitude': 'lon', 'latitude': 'lat', 't2m': 'temperature'})
-    ds = ds.assign_coords({'height': 2})
-    ds = ds.expand_dims('height')
-    ds['temperature'] = ds['temperature'] - 273.15
-    ds['temperature'].encoding = encoding
+#     """
+#     ds = ds.rename({'longitude': 'lon', 'latitude': 'lat', 't2m': 'temperature'})
+#     ds = ds.assign_coords({'height': 2})
+#     ds = ds.expand_dims('height')
+#     ds['temperature'] = ds['temperature'] - 273.15
+#     ds['temperature'].encoding = encoding
 
-    return ds
+#     return ds
 
 # p1 = Processor(data_path, parameter_code, False)
 # ds1 = p1.build_dataset()
@@ -134,11 +136,11 @@ samples=10
 # chunks = {'time': 10000, 'latitude': 10, 'longitude': 10}
 chunks = {'time': 10000}
 
-self = Grid()
+self = Processor()
 
-ds_list = self.load_grid_metadata(datasets)
+ds_list = self.load_dataset_metadata(datasets)
 
-grid1 = self.load_raw_grid(data_path2, lon_dim_name, lat_dim_name, time_dim_name, preprocessor=preprocessor)
+grid1 = self.load_raw_grid(data_path2, lon_dim_name, lat_dim_name, time_dim_name, preprocessor=tu.grid.processors.era5_land.preprocessor)
 
 
 
@@ -213,7 +215,7 @@ ds = xr.open_dataset(NetCDF4DataStore(nc))
 
 
 
-ds = open_wrf_dataset(data_path1)
+ds = xr.open_mfdataset(data_path2)
 
 
 
@@ -267,17 +269,18 @@ self = Processor()
 
 ds_list = self.load_dataset_metadata(datasets)
 
-grid1 = self.load_raw_grid(data_path2, lon_dim_name, lat_dim_name, time_dim_name, preprocessor=preprocessor)
+# grid1 = self.load_raw_grid(data_path2, lon_dim_name, lat_dim_name, time_dim_name, preprocessor=tu.grid.processors.era5_land.preprocessor)
+grid1 = self.load_raw_grid(data_path4, lon_dim_name, lat_dim_name, time_dim_name, preprocessor=preprocessor)
 
 obj_dict = self.determine_grid_block_size(ds_list[0]['dataset_id'])
 
-self.save_grid_chunks(output_path)
+# self.save_grid_chunks(output_path)
 
 self.save_grid_chunks(output_path, 1)
 
 
 
-self = tu.grid.Grid()
+self = Grid()
 
 ds_list = self.load_dataset_metadata(datasets)
 
@@ -304,7 +307,7 @@ self.load_results(results, xy_size, dataset_id, sum_closed, other_closed, discre
 
 # self.public_url = public_url
 
-self.update_results(threads=10)
+self.update_results(threads=20)
 
 self.update_aggregates()
 
