@@ -10,7 +10,7 @@ import numpy as np
 import xarray as xr
 import pandas as pd
 from tethys_utils import misc, s3, processing, titan
-from tethysts.utils import s3_client, get_object_s3, read_json_zstd, read_pkl_zstd, download_results, create_public_s3_url
+from tethysts.utils import read_json_zstd, read_pkl_zstd, download_results, create_public_s3_url
 import tethys_data_models as tdm
 from typing import List, Optional, Dict, Union
 import uuid
