@@ -10,7 +10,7 @@ import numpy as np
 import xarray as xr
 import pandas as pd
 from tethys_utils import misc, s3, processing
-from tethysts.utils import s3_client, get_object_s3, read_json_zstd, read_pkl_zstd, create_public_s3_url, make_run_date_key
+from tethysts.utils import read_json_zstd, read_pkl_zstd, create_public_s3_url, make_run_date_key
 import tethys_data_models as tdm
 from typing import List, Optional, Dict, Union
 from botocore import exceptions as bc_exceptions
@@ -192,7 +192,7 @@ class Titan(object):
         """
         ## Test S3 connection
         _ = tdm.base.Remote(**{'connection_config': connection_config, 'bucket': bucket, 'version': system_version})
-        client = s3_client(connection_config)
+        client = misc.s3_client(connection_config)
 
         # Read permissions
         try:
@@ -217,7 +217,7 @@ class Titan(object):
         if isinstance(public_url, str):
             try:
                 _ = tdm.base.Remote(**{'public_url': public_url, 'bucket': bucket, 'version': system_version})
-                _ = get_object_s3(test_key, public_url=public_url, bucket=bucket, counter=1)
+                _ = misc.get_object_s3(test_key, public_url=public_url, bucket=bucket, counter=1)
             except:
                 raise ValueError('public_url does not work.')
 
@@ -256,7 +256,7 @@ class Titan(object):
             dgn_key = dgn_key_pattern.format(run_id=self.run_id)
 
             try:
-                resp = get_object_s3(dgn_key, bucket, client, public_url=public_url, counter=0)
+                resp = misc.get_object_s3(dgn_key, bucket, client, public_url=public_url, counter=0)
                 f = Fernet(self.key)
                 dgn = read_json_zstd(f.decrypt(resp))
                 self.diagnostics = dgn
@@ -284,7 +284,7 @@ class Titan(object):
         dgn_key_pattern = tdm.key_patterns[self.system_version]['diagnostics']
         dgn_key = dgn_key_pattern.format(run_id=self.run_id)
 
-        client = s3_client(self.connection_config)
+        client = misc.s3_client(self.connection_config)
         run_date1 = misc.make_run_date_key()
 
         obj_resp = s3.put_object_s3(client, self.bucket, dgn_key, dgn_obj, {'run_date': run_date1}, 'application/json')
@@ -693,7 +693,7 @@ class Titan(object):
         # Get the old file
         dss_key = tdm.utils.key_patterns[self.system_version]['datasets']
 
-        dss_obj = get_object_s3(dss_key, bucket=self.bucket, connection_config=self.connection_config, public_url=self.public_url, counter=2)
+        dss_obj = misc.get_object_s3(dss_key, bucket=self.bucket, connection_config=self.connection_config, public_url=self.public_url, counter=2)
 
         if dss_obj is None:
             old_datasets = None
@@ -718,7 +718,7 @@ class Titan(object):
 
         if len(rc_paths) > 0:
 
-            results_paths = glob.glob(os.path.join(self.preprocessed_path, '*results.nc.zst'))
+            results_paths = glob.glob(os.path.join(self.preprocessed_path, '*results.h5'))
             stns_paths = glob.glob(os.path.join(self.preprocessed_path, '*stations.json.zst'))
             v_paths = glob.glob(os.path.join(self.preprocessed_path, '*versions.json.zst'))
             ds_paths = glob.glob(os.path.join(self.preprocessed_path, '*dataset.json.zst'))
@@ -747,7 +747,7 @@ class Titan(object):
             all_paths.extend(dss_path)
             random.shuffle(all_paths)
 
-            client = s3_client(self.connection_config, max_pool_connections=threads)
+            client = misc.s3_client(self.connection_config, max_pool_connections=threads)
 
             s3_list = []
             append = s3_list.append
@@ -881,7 +881,7 @@ class Titan(object):
 
         key_prefix = tdm.utils.key_patterns[self.system_version]['results'].split('{station_id}')[0].format(dataset_id=dataset_id, version_date=vd_key)
 
-        client = s3_client(self.connection_config, threads)
+        client = misc.s3_client(self.connection_config, threads)
 
         obj_list1 = s3.list_objects_s3(client, self.bucket, key_prefix)
         obj_list1 = obj_list1[obj_list1.Key.str.contains('results')].copy()
